// -------------------------------------------------------------
// variables that are provided by the application
// -------------------------------------------------------------
half4x4 matWorldViewProj;
half4x4 matHeadWorldViewProj;
half4 vecHeadLight;
half4 vecLight;
half4 vecView;

texture texBase;
sampler baseSampler = sampler_state
{
   Texture = <texBase>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture texShift;
sampler shiftSampler = sampler_state
{
   Texture = <texShift>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture texSpecular;
sampler specularSampler = sampler_state
{
   Texture = <texSpecular>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture texAlpha;
sampler alphaSampler = sampler_state
{
   Texture = <texAlpha>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture texDatabase;
sampler databaseSampler = sampler_state
{
   Texture = <texDatabase>;
   MinFilter = Anisotropic;
   MagFilter = Anisotropic;
   MipFilter = Anisotropic;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture texShadow;

sampler shadowSampler = sampler_state
{
   Texture = <texShadow>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

struct VS0_OUTPUT
{
   half4 Pos: POSITION;
   half3 Nor: TEXCOORD0;
   half3 Light : TEXCOORD1;
};

VS0_OUTPUT VS0( half4 Pos: POSITION, half3 Nor: NORMAL)
{
	VS0_OUTPUT Out;
	Out.Pos = mul(Pos, matHeadWorldViewProj); // transform Position
	Out.Nor = normalize(Nor); 
	Out.Light = normalize(vecHeadLight);
	return Out;
}

half4 PS0(half3 Nor: TEXCOORD0, half3 Light: TEXCOORD1): COLOR
{
	half3 diffuse = {241.0/256.0, 207.0/256.0, 196.0/256.0};
	diffuse = saturate(diffuse * dot(Light, Nor));
	return half4(diffuse.x, diffuse.y, diffuse.z, 1);
}


// -------------------------------------------------------------
// Output channels
// -------------------------------------------------------------
struct VS_OUTPUT
{
   half4 Pos: POSITION;
   half3 View: TEXCOORD0;
   half3 Light: TEXCOORD1;
   half2 UV: TEXCOORD2;
   half3 Nor: TEXCOORD3;
   half3 Tan: TEXCOORD4;

};

// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------
VS_OUTPUT VS1( half4 Pos: POSITION, half3 Nor: NORMAL, half2 UV: TEXCOORD0, half3 Tan: TEXCOORD1)
{
	VS_OUTPUT Out = (VS_OUTPUT) 0;
	Out.Pos = mul(Pos, matWorldViewProj); // transform Position

	// make Tan and Nor perpendicular to each other
	half3 vecTmp;
	vecTmp = cross(Tan, Nor);
	Nor = cross(vecTmp, Tan);

	Out.View = normalize(vecView);
	Out.Light = normalize(vecLight);
	Out.UV = UV;
	Out.Nor = normalize(Nor);
	Out.Tan = normalize(Tan);
	return Out;
}

// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
half4 PS1(half3 View: TEXCOORD0, half3 Light: TEXCOORD1, half2 UV: TEXCOORD2, half3 Nor: TEXCOORD3, half3 Tan: TEXCOORD4) : COLOR
{
	half3 diffuse = {0.8, 0.64, 0.45};
	half3 specular1 = {0.5, 0.5, 0.5};
	half3 specular2 = {0.4, 0.32, 0.225};
	
	half shift = 0.4f*(tex2D(shiftSampler, UV) - 0.5);
	half3 t1 = normalize(Tan + (shift + 0.075) * Nor);
	half3 t2 = normalize(Tan + (shift - 0.075) * Nor);
	
	diffuse = 1.4*diffuse*sqrt(1 - dot(Light, Tan)*dot(Light, Tan));

	//Specular = [sin(T,L)sin(T,E) + T.L * T.E]^p = [sqrt(1 C (T.L)^2) sqrt(1 C (T.E)^2) - T.L * T.E]^p
	half temp1 = dot(t1, Light);
	half temp2 = dot(t1, View);
	
	//primary highlight
	half s1 = sqrt(1-temp1*temp1)*sqrt(1-temp2*temp2) - temp1*temp2;
	specular1 = 0.8f*specular1*pow(s1, 50.0f);
	
	temp1 = dot(t2, Light);
	temp2 = dot(t2, View);

	//secondary highlight
	half s2 = sqrt(1-temp1*temp1)*sqrt(1-temp2*temp2) - temp1*temp2;
	half specMask = tex2D(specularSampler, UV);
	specular2 = 1.2f*specular2*specMask*pow(s2, 50.0f);

	half3 color;
	half4 tmp = tex2D(databaseSampler, UV);
	half c = (tmp.x+0.2)-tex2D(shadowSampler, UV).x*4*(tmp.x+0.2);

	color = saturate((diffuse + specular1 + specular2)* c * 2.2);

	return half4(color.xyz, tmp.a);
}

// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------
VS_OUTPUT VS2( half4 Pos: POSITION, half3 Nor: NORMAL, half2 UV: TEXCOORD0, half3 Tan: TEXCOORD1)
{
	VS_OUTPUT Out = (VS_OUTPUT) 0;
	Out.Pos = mul(Pos, matWorldViewProj); // transform Position

	// make Tan and Nor perpendicular to each other
	half3 vecTmp;
	vecTmp = cross(Tan, Nor);
	Nor = cross(vecTmp, Tan);

	Out.View = normalize(vecView);
	Out.Light = normalize(vecLight);
	Out.UV = UV;
	Out.Nor = normalize(Nor);
	Out.Tan = normalize(Tan);
	return Out;
}

// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
half4 PS2(half3 View: TEXCOORD0, half3 Light: TEXCOORD1, half2 UV: TEXCOORD2, half3 Nor: TEXCOORD3, half3 Tan: TEXCOORD4) : COLOR
{
	half3 diffuse = {181.0/256.0, 116.0/256.0, 96.0/256.0};
	half3 specular1 = {0.5, 0.5, 0.5};
	half3 specular2 = {121.0/256.0, 77.0/256.0, 64.0/256.0};
	
	half shift = 0.4f*(tex2D(shiftSampler, UV) - 0.5);
	half3 t1 = normalize(Tan + (shift + 0.075) * Nor);
	half3 t2 = normalize(Tan + (shift - 0.075) * Nor);
	
	diffuse = 1.4*diffuse*sqrt(1 - dot(Light, Tan)*dot(Light, Tan));

	//Specular = [sin(T,L)sin(T,E) + T.L * T.E]^p = [sqrt(1 C (T.L)^2) sqrt(1 C (T.E)^2) - T.L * T.E]^p
	half temp1 = dot(t1, Light);
	half temp2 = dot(t1, View);
	
	//primary highlight
	half s1 = sqrt(1-temp1*temp1)*sqrt(1-temp2*temp2) - temp1*temp2;
	specular1 = 0.8f*specular1*pow(s1, 50.0f);
	
	temp1 = dot(t2, Light);
	temp2 = dot(t2, View);

	//secondary highlight
	half s2 = sqrt(1-temp1*temp1)*sqrt(1-temp2*temp2) - temp1*temp2;
	half specMask = tex2D(specularSampler, UV);
	specular2 = 1.2f*specular2*specMask*pow(s2, 50.0f);

	half3 color;
	half4 tmp = tex2D(databaseSampler, UV);
	half c = (tmp.x+0.2)-tex2D(shadowSampler, UV).x*6*(tmp.x+0.2);

	color = saturate((diffuse + specular1 + specular2)* c * 2.2);

	return half4(color.xyz, tmp.a);
}

// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------
VS_OUTPUT VS3( half4 Pos: POSITION, half3 Nor: NORMAL, half2 UV: TEXCOORD0, half3 Tan: TEXCOORD1)
{
	VS_OUTPUT Out = (VS_OUTPUT) 0;
	Out.Pos = mul(Pos, matWorldViewProj); // transform Position

	// make Tan and Nor perpendicular to each other
	half3 vecTmp;
	vecTmp = cross(Tan, Nor);
	Nor = cross(vecTmp, Tan);

	Out.View = normalize(vecView);
	Out.Light = normalize(vecLight);
	Out.UV = UV;
	Out.Nor = normalize(Nor);
	Out.Tan = normalize(Tan);
	return Out;
}

// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
half4 PS3(half3 View: TEXCOORD0, half3 Light: TEXCOORD1, half2 UV: TEXCOORD2, half3 Nor: TEXCOORD3, half3 Tan: TEXCOORD4) : COLOR
{
	half3 diffuse = {106.0/256.0, 142.0/256.0, 169.0/256.0};
	half3 specular1 = {0.5, 0.5, 0.5};
	half3 specular2 = {71.0/256.0, 95.0/256.0, 116.0/256.0};
	
	half shift = 0.4f*(tex2D(shiftSampler, UV) - 0.5);
	half3 t1 = normalize(Tan + (shift + 0.075) * Nor);
	half3 t2 = normalize(Tan + (shift - 0.075) * Nor);
	
	diffuse = 1.4*diffuse*sqrt(1 - dot(Light, Tan)*dot(Light, Tan));

	//Specular = [sin(T,L)sin(T,E) + T.L * T.E]^p = [sqrt(1 C (T.L)^2) sqrt(1 C (T.E)^2) - T.L * T.E]^p
	half temp1 = dot(t1, Light);
	half temp2 = dot(t1, View);
	
	//primary highlight
	half s1 = sqrt(1-temp1*temp1)*sqrt(1-temp2*temp2) - temp1*temp2;
	specular1 = 0.8f*specular1*pow(s1, 50.0f);
	
	temp1 = dot(t2, Light);
	temp2 = dot(t2, View);

	//secondary highlight
	half s2 = sqrt(1-temp1*temp1)*sqrt(1-temp2*temp2) - temp1*temp2;
	half specMask = tex2D(specularSampler, UV);
	specular2 = 1.2f*specular2*specMask*pow(s2, 50.0f);

	half3 color;
	half4 tmp = tex2D(databaseSampler, UV);
	half c = (tmp.x+0.2)-tex2D(shadowSampler, UV).x*6*(tmp.x+0.2);

	color = saturate((diffuse + specular1 + specular2)* c * 2.2);

	return half4(color.xyz, tmp.a);
}

// -------------------------------------------------------------
// vertex shader function (input channels)
// -------------------------------------------------------------
VS_OUTPUT VS4( half4 Pos: POSITION, half3 Nor: NORMAL, half2 UV: TEXCOORD0, half3 Tan: TEXCOORD1)
{
	VS_OUTPUT Out = (VS_OUTPUT) 0;
	Out.Pos = mul(Pos, matWorldViewProj); // transform Position

	// make Tan and Nor perpendicular to each other
	half3 vecTmp;
	vecTmp = cross(Tan, Nor);
	Nor = cross(vecTmp, Tan);

	Out.View = normalize(vecView);
	Out.Light = normalize(vecLight);
	Out.UV = UV;
	Out.Nor = normalize(Nor);
	Out.Tan = normalize(Tan);
	return Out;
}

// -------------------------------------------------------------
// Pixel Shader (input channels):output channel
// -------------------------------------------------------------
half4 PS4(half3 View: TEXCOORD0, half3 Light: TEXCOORD1, half2 UV: TEXCOORD2, half3 Nor: TEXCOORD3, half3 Tan: TEXCOORD4) : COLOR
{
	half3 diffuse = {0.3, 0.3, 0.3};
	half3 specular1 = {0.5, 0.5, 0.5};
	half3 specular2 = {0.4, 0.32, 0.225};
	
	half shift = 0.4f*(tex2D(shiftSampler, UV) - 0.5);
	half3 t1 = normalize(Tan + (shift + 0.075) * Nor);
	half3 t2 = normalize(Tan + (shift - 0.075) * Nor);
	
	diffuse = 1.4*diffuse*sqrt(1 - dot(Light, Tan)*dot(Light, Tan));

	//Specular = [sin(T,L)sin(T,E) + T.L * T.E]^p = [sqrt(1 C (T.L)^2) sqrt(1 C (T.E)^2) - T.L * T.E]^p
	half temp1 = dot(t1, Light);
	half temp2 = dot(t1, View);
	
	//primary highlight
	half s1 = sqrt(1-temp1*temp1)*sqrt(1-temp2*temp2) - temp1*temp2;
	specular1 = 0.8f*specular1*pow(s1, 50.0f);
	
	temp1 = dot(t2, Light);
	temp2 = dot(t2, View);

	//secondary highlight
	half s2 = sqrt(1-temp1*temp1)*sqrt(1-temp2*temp2) - temp1*temp2;
	half specMask = tex2D(specularSampler, UV);
	specular2 = 1.2f*specular2*specMask*pow(s2, 50.0f);

	half3 color;
	half4 tmp = tex2D(databaseSampler, UV);
	half c = (tmp.x+0.2)-tex2D(shadowSampler, UV).x*6*(tmp.x+0.2);

	color = saturate((diffuse + specular1 + specular2)* c * 2.2);

	return half4(color.xyz, tmp.a);
}


half4 VS_SH( half4 Pos: POSITION) : POSITION
{
	return mul(Pos, matWorldViewProj); // transform Position
}

half4 PS_SH(): COLOR
{
	return half4(0, 0, 0, 0.5);
}



// -------------------------------------------------------------
// Name technique | how to compile 
// -------------------------------------------------------------
technique TShader
{
	//head
	pass P0
    {
        VertexShader = compile vs_1_0 VS0();
        PixelShader  = compile ps_2_0 PS0();
    }
    pass P1
    {
        // compiler directives
        VertexShader = compile vs_1_0 VS1();
        PixelShader  = compile ps_2_0 PS1();
    }
	pass P2
    {
        // compiler directives
        VertexShader = compile vs_1_0 VS2();
        PixelShader  = compile ps_2_0 PS2();
    }
	pass P3
    {
        // compiler directives
        VertexShader = compile vs_1_0 VS3();
        PixelShader  = compile ps_2_0 PS3();
    }
	pass P4
    {
        // compiler directives
        VertexShader = compile vs_1_0 VS4();
        PixelShader  = compile ps_2_0 PS4();
    }

	pass P5
	{
		VertexShader = compile vs_1_0 VS_SH();
        PixelShader  = compile ps_1_0 PS_SH();
	}

}
